unit Form1;

// DISCLAIMER!
// ==================================================================
// This source code has been automatically generated.
// It still requires extensive manual fixing, debugging and testing.
// IT IS NOT GUARANTEED TO COMPILE
// AND MUCH LESS TO BEHAVE EXACTLY LIKE THE ORIGINAL SOURCE CODE.
// Use it at your sole risk and discretion.
//

interface

  uses
    Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
    Dialogs;

  type
    TForm1 =
      class( TForm )
        chkPrompt : TCheckBox;
        Dir1 : TDirectoryListBox;
        Drive1 : TDriveComboBox;
        File1 : TFileListBox;
        Form1 : TForm;
        Label1 : TLabel;
        private
          procedure Form_Load;
          procedure Drive1_Change;
          procedure Dir1_Change;
          procedure Dir1_MouseUp( var Button : smallint; var Shift : smallint; var x : single; var y : single );
          procedure File1_KeyDown( var KeyCode : smallint; var Shift : smallint );
          procedure File1_MouseUp( var Button : smallint; var Shift : smallint; var x : single; var y : single );
          procedure ShellContextMenu( var objLB : Control; var x : single; var y : single; var Shift : smallint );
          function GetFileLBItemPath( var objFLB : TFileListBox; var iItem : smallint ) : widestring;
          function GetDirLBItemPath( var objDLB : TDirectoryListBox; var iItem : smallint ) : widestring;
          procedure RefreshListBox( var objLB : Control );
        public
      end;

implementation

  {$R *.DFM}

  //
  // Brought to you by Brad Martinez
  //   http://members.aol.com/btmtz/vb
  //   http://www.mvps.org/ccrp
  //
  // Code was written in and formatted for 8pt MS San Serif
  //
  // Demonstrates how to show the shell context menu for any directory or
  // group of files in the file system. Also demonstrates how to navigate pidls
  // (pointers to item ID lists)
  //
  // Version 1.30
  //
  // Note that "IShellFolder Extended Type Library v1.2" (ISHF_Ex.tlb)
  // included with this project, must be present and correctly registered
  // on your system, and referenced by this project, to allow use of the
  // IShellFolder, IContextMenu and IMalloc interfaces. ** Be aware that
  // this type library is a newer version than one included with previous
  // version of this demo project.**
  //
  // v1.10 update:
  //   - Corrected FileListBox selection bug
  //   - Shows how to retrieve the selected context menu command's string.
  //   - Added the option to prompt before executing the selected command.
  //
  // v1.20 update:
  //   - Corrected the "extended" FileListBox selection bug, again.
  //   - Shows how insert and execute a user-defined menu command in the
  //     context menu.
  //   - Did a little more documenting...
  //
  // v1.30 update:
  //   - Added IContextMenu2 support, allowing the "Send To" and "Open With"
  //     submenus to be filled with their respective items.
  //   - Added subclassing module to catch ownerdraw menu messages for above.
  //   - Now inserts the focused FileListBox file as the first element of the array of
  //     relative pidls passed to ShowShellContextMenu. This allows the context
  //     menu to contain the commands for this file when multiple files are selected.
  //   - Both listboxes now refresh after each context menu command is carried out.
  //
  procedure TForm1.Form_Load;
    begin
      //  File1.MultiSelect = 2
      Move( ( Screen.Width - Width ) * 0.5, ( Screen.Height - Height ) * 0.5 );
    end;

  procedure TForm1.Drive1_Change;
    label
      Out;
    begin
      //ON ERROR
      Dir1.Directory := Drive1.Drive;
      Out:
    end;

  procedure TForm1.Dir1_Change;
    label
      Out;
    begin
      //ON ERROR
      File1 := Dir1.Directory;
      Out:
    end;

  procedure TForm1.Dir1_MouseUp( var Button : smallint; var Shift : smallint; var x : single; var y : single );
    begin
      if ( Button = vbRightButton )
        then
          begin
            ShellContextMenu( Dir1, x, y, Shift );
          end;
    end;

  // Selects all FileListBox items on a Ctrl+A keypress.
  procedure TForm1.File1_KeyDown( var KeyCode : smallint; var Shift : smallint );
    begin
      if ( KeyCode = vbKeyA ) and ( Shift = vbCtrlMask )
        then
          begin
            SendMessage( File1.hWnd, LB_SETSEL, CTrue, -1 );
          end;
    end;

  procedure TForm1.File1_MouseUp( var Button : smallint; var Shift : smallint; var x : single; var y : single );
    begin
      if ( Button = vbRightButton )
        then
          begin
            ShellContextMenu( File1, x, y, Shift );
          end;
    end;

  procedure TForm1.ShellContextMenu( var objLB : Control; var x : single; var y : single; var Shift : smallint );
    var
      pt : POINTAPI;  // screen location of the cursor
      iItem : smallint;  // listbox index of the selected item (item under the cursor)
      cItems : smallint;  // count of selected items
      i : smallint;  // counter
      asPaths : TStringDynArray;  // array of selected items' paths (zero based)
      apidlFQs : TIntegerDynArray;  // array of selected items' fully qualified pidls (zero based)
      isfParent : IShellFolder;  // selected items' parent shell folder
      apidlRels : TIntegerDynArray;  // array of selected items' relative pidls (zero based)
    begin
      // ==================================================
      // Get the listbox item under the cursor
      // Convert the listbox's client twip coords to screen pixel coords.
      pt.x := x div Screen.TwipsPerPixelX;
      pt.y := y div Screen.TwipsPerPixelY;
      ClientToScreen( objLB.hWnd, pt );
      // Get the zero-based index of the item under the cursor.
      // If none exists, bail...
      iItem := LBItemFromPt( objLB.hWnd, pt.x, pt.y, false );
      if ( iItem = LB_ERR )
        then
          begin
            exit;
          end;
      // ==================================================
      // Set listbox focus and selection
      objLB.SetFocus;
      // If neither the Control and/or Shift key are pressed...
      if ( ( Shift and ( vbCtrlMask or vbShiftMask ) ) = false )
        then
          begin
            // If Dir1 has the focus...
            if ( objLB is DirListBox )
              then
                begin
                  // Select the item under the cursor. The DirListBox
                  // doesn't have a Selected property, so we'll get forceful...
                  SendMessage( Dir1.hWnd, LB_SETCURSEL, iItem, 0 );
                end
              else
                begin
                  // File1 has the focus, duplicate Explorer listview selection functionality.
                  // If the right clicked item isn't selected....
                  if ( File1.Selected( iItem ) = false )
                    then
                      begin
                        // Deselect all of the items and select the right clicked item.
                        SendMessage( File1.hWnd, LB_SETSEL, CFalse, -1 );
                        File1.Selected[iItem] := true;
                      end
                    else
                      begin
                        // The right clciked item is selected, give it the selection rectangle
                        // (or caret, does not deselect any other currently selected items).
                        // File1.Selected doesn't set the caret if the item is already selected.
                        SendMessage( File1.hWnd, LB_SETCARETINDEX, iItem, 0 );
                      end;
                end;
          end; // (Shift And (vbCtrlMask Or vbShiftMask)) = False
      // ==================================================
      // Load the path(s) of the selected listbox item(s) into the array.
      if ( objLB is DirListBox )
        then
          begin
            // Only one directory can be selected in the DirLB
            cItems := 1;
            setlength( asPaths, 0 );
            asPaths[0] := GetDirLBItemPath( Dir1, iItem );
          end
        else
          begin
            // Put the focused (and selected) files's relative pidl in the
            // first element of the array. This will be the file whose context
            // menu will be shown if multiple files are selected.
            cItems := 1;
            setlength( asPaths, 0 );
            asPaths[0] := GetFileLBItemPath( File1, iItem );
            // Fill the array with the relative pidls of the rest of any selected
            // files(s), making sure that we don't add the focused file again.
            for i := 0 to File1.ListCount - 1 do
              begin
                if ( File1.Selected( i ) ) and ( i <> iItem )
                  then
                    begin
                      cItems := cItems + 1;
                      setlength( asPaths, cItems - 1 );
                      asPaths[cItems - 1] := GetFileLBItemPath( File1, i );
                    end;
              end;
          end;
      // ==================================================
      // Finally, get the IShellFolder of the selected directory, load the relative
      // pidl(s) of the selected items into the array, and show the menu.
      // This part won't be elaborated upon, as it is extensively involved.
      // For more info on IShellFolder, pidls and the shell's context menu, see:
      // http://msdn.microsoft.com/developer/sdk/inetsdk/help/itt/Shell/NameSpace.htm
      if length( asPaths[0] )
        then
          begin
            // Get a copy of each selected item's fully qualified pidl from it's path.
            for i := 0 to cItems - 1 do
              begin
                setlength( apidlFQs, i );
                apidlFQs[i] := GetPIDLFromPath( hWnd, asPaths[i] );
              end;
            if apidlFQs[0]
              then
                begin
                  // Get the selected item's parent IShellFolder.
                  isfParent := GetParentIShellFolder( apidlFQs[0] );
                  if ( ( isfParent = nil ) = false )
                    then
                      begin
                        // Get a copy of each selected item's relative pidl (the last item ID)
                        // from each respective item's fully qualified pidl.
                        for i := 0 to cItems - 1 do
                          begin
                            setlength( apidlRels, i );
                            apidlRels[i] := GetItemID( apidlFQs[i], GIID_LAST );
                          end;
                        if apidlRels[0]
                          then
                            begin
                              // Subclass the Form so we catch the menu's ownerdraw messages.
                              SubClass( hWnd, @WndProc );
                              // Show the shell context menu for the selected items. If a
                              // menu command was executed, refresh the two listboxes.
                              if ShowShellContextMenu( hWnd, isfParent, cItems, apidlRels[0], pt, chkPrompt )
                                then
                                  begin
                                    Dir1.Refresh;
                                    RefreshListBox( File1 );
                                  end;
                              // Finally, unsubclass the form.
                              UnSubClass( hWnd );
                            end; // apidlRels(0)
                        // Free each item's relative pidl.
                        for i := 0 to cItems - 1 do
                          begin
                            MemAllocator.Free( apidlRels[i] );
                          end;
                      end; // (isfParent Is Nothing) = False
                  // Free each item's fully qualified pidl.
                  for i := 0 to cItems - 1 do
                    begin
                      MemAllocator.Free( apidlFQs[i] );
                    end;
                end; // apidlFQs(0)
          end; // Len(asPaths(0))
    end;

  function TForm1.GetFileLBItemPath( var objFLB : TFileListBox; var iItem : smallint ) : widestring;
    var
      sPath : widestring; 
    begin
      sPath := objFLB.Path;
      if ( RightStr(sPath, 1) <> '\' )
        then
          begin
            sPath := sPath + '\';
          end;
      result := sPath + objFLB.List( iItem );
    end;

  // Returns the DirListBox Path from the specified listbox item index.
  //   - the currently expanded directory (lowest in hierarchy) is ListIndex -1
  //   - it's 1st parent directory's ListIndex is -2, if any (the parent indices get smaller)
  //   - it's 1st child subdirectory's ListIndex is 0, if any (the child indices get larger)
  //   - ListCount is the number of child subdirectories under the currently expanded directory.
  //   - List(x) returns the full path of item whose index is x
  //   - there is never more than one expanded directory on any directory hierachical level
  // It's a little extra work getting the path of the selected DirListBox item...
  function TForm1.GetDirLBItemPath( var objDLB : TDirectoryListBox; var iItem : smallint ) : widestring;
    var
      nItems : smallint; 
    begin
      // Get the count of items in the DirLB
      nItems := SendMessage( objDLB.hWnd, LB_GETCOUNT, 0, 0 );
      if ( nItems > -1 ) // LB_ERR
        then
          begin
            // Subtract the actual number of LB items from the sum of:
            //   the DirLB's ListCount and
            //   the currently selected directory's real LB index value
            // (nItems is a value of 1 greater than the last item's real LB index value)
            result := objDLB.List( ( objDLB.Count + iItem ) - nItems );
            //Debug.Print "iItem: " & iItem & ", LiistIndex: " & (objDLB.ListCount + iItem) - nItems
          end;
    end;

  procedure TForm1.RefreshListBox( var objLB : Control );
    var
      iFocusedItem : smallint; 
      i : smallint; 
      cItems : smallint; 
      aiSelitems : TSmallIntDynArray; 
    begin
      // Cache the focused item, if any.
      iFocusedItem := objLB.ListIndex;
      // Cache any selected items
      for i := 0 to objLB.ListCount - 1 do
        begin
          if objLB.Selected( i )
            then
              begin
                cItems := cItems + 1;
                setlength( aiSelitems, cItems - 1 );
                aiSelitems[cItems - 1] := i;
              end;
        end;
      // Refresh the listbox, sets ListIndex = 0, and removes all selction.
      objLB.Refresh;
      // Restore focus and selection to the cached items.
      //  objLB.ListIndex = iFocusedItem   ' this errs... (?)
      SendMessage( objLB.hWnd, LB_SETCARETINDEX, iFocusedItem, 0 );
      for i := 0 to cItems - 1 do
        begin
          //    objLB.Selected(aiSelitems(i)) = True   ' may err...
          SendMessage( objLB.hWnd, LB_SETSEL, CTrue, aiSelitems[i] );
        end;
    end;

end.
