Attribute VB_Name = "MComCtrlInit"
Option Explicit

' Brought to you by:
'   Brad Martinez
'   btmtz@msn.com
'   btmtz@aol.com
'   http://members.aol.com/btmtz/vb

' ========================================================
' Prototypes were extracted and translated from Commctrl.h v1.2
' which can be found at:
' http://www.microsoft.com/msdn/sdk/platforms/doc/sdk/headers/c35.htm

' Descriptions and syntax were extracted from the Win32 SDK
' also available at: http://www.microsoft.com/win32dev/wcnt.htm

' Both URLs were valid 06/19/97, the time of this writing.
' ========================================================

' Initializes the entire common control dynamic-link library.
' Exported by all versions of Comctl32.dll.
Declare Sub InitCommonControls Lib "comctl32.dll" ()

' Initializes specific common controls classes from the common
' control dynamic-link library.
' Returns TRUE (non-zero) if successful, or FALSE otherwise.
' Began being exported with Comctl32.dll version 4.7 (IE3.0 & later).
Declare Function InitCommonControlsEx Lib "comctl32.dll" _
                            (lpInitCtrls As tagINITCOMMONCONTROLSEX) As Boolean

Type tagINITCOMMONCONTROLSEX   ' icc
    dwSize As Long   ' size of this structure
    dwICC As Long    ' flags indicating which classes to be initialized.
End Type

' Set of bit flags that indicate which common control classes will be loaded
' from the DLL. The dwICC value can be a combination of the following:

Public Const ICC_LISTVIEW_CLASSES = &H1    ' listview, header
Public Const ICC_TREEVIEW_CLASSES = &H2  ' treeview, tooltips
Public Const ICC_BAR_CLASSES = &H4            ' toolbar, statusbar, trackbar, tooltips
Public Const ICC_TAB_CLASSES = &H8             ' tab, tooltips
Public Const ICC_UPDOWN_CLASS = &H10       ' updown
Public Const ICC_PROGRESS_CLASS = &H20   ' progress bar
Public Const ICC_HOTKEY_CLASS = &H40         ' hotkey
Public Const ICC_ANIMATE_CLASS = &H80        ' animate control class
Public Const ICC_WIN95_CLASSES = &HFF        ' loads everything above
Public Const ICC_DATE_CLASSES = &H100        ' month picker, date picker, time picker, updown
Public Const ICC_USEREX_CLASSES = &H200   ' comboex
Public Const ICC_COOL_CLASSES = &H400       ' rebar (coolbar) control
 

' Returns True if the current working version of Comctl32.dll is
' available on the system and the new IE3 sytles and messages
' can be used. Returns False either if the old version is present or
' Comctl32.dll isn't available at all. Also ensures that the library is
' loaded for use.

' We can get away with this hack rather than checking the file's
' version because VB resolves declared API function names only
' when they're called, not when it compiles the code...

Public Function InitComctl32(dwFlags As Long) As Boolean

  Dim icc As tagINITCOMMONCONTROLSEX
  On Error GoTo Err_OldVersion
  
  icc.dwSize = Len(icc)
  icc.dwICC = dwFlags
  
  ' VB will generate error 453 "Specified DLL function not found"
  ' here if the new version isn't installed and it can't find the function's
  ' name. We'll hopefully be able to load the old version below...
  InitComctl32 = InitCommonControlsEx(icc)
  Exit Function

Err_OldVersion:
  InitCommonControls

End Function
